function [l_aux, mz] = calc_aux_metric(f, z, m, l)
% calc_aux_metric
% Get the auxiliary metric for f and l, associated with the beltrami
% coefficient m (calculation by section 5 in paper [1]).
% Input:
%   f : an array of dimension (number of triangles) x 3. 
%       f(m,n) is the index of the nth vertex in the mth triangle.
%   z : an array of dimension (number of vertices) x 2.
%       z(m,:) are the coordinates of the mth vertex of the discrete
%       conformally flattened  mesh.
%    m: a function handle, which is the beltrami coefficient
%    l: the original mesh's edge lengths
%
% Output:
%   l_aux :  a matrix of size  (number of vertices) x (number of vertices).
%          This is the auxilary metric for the given metric l on the
%          mesh asspciated with the beltrami coefficient m.
%   mz : array of dimension  (number of triangles) x 1.
%           mz(m) is the mean of the coordinates of the image of m 
%           on the m'th vertex in z.  

% get derivative of conformal param' on edges
 dz =  z(f(:, [2 3 1]), :) - z(f(:, [3 1 2]), :);

 % get beltrami coefficient on edges
 cz = z(:,1) + 1i*z(:,2);
 mz = m(cz);
 if (size(mz) == 1)
     mz = ones(size(cz)) * mz;
 end
 
 meu = 1/2 * (mz(f(:, [2 3 1])) + mz(f(:, [3 1 2])));
 
 % get mz for coloring the flat mesh later.
 % we need only a scalar value per vertex, so take the mean of 
 % the two coordinates.
 mz = abs(mz);
 
 % get complex conjugate of dz
 cdz = dz(:,1) + 1i*dz(:,2);
 dz_bar = conj(cdz);
 dz_bar = reshape(dz_bar,[],3);
 dzmeu = dz_bar .* meu;
 
 cdz = reshape(cdz,[],3);
 
 % get aux metric scalar
 lambda = (abs(dzmeu(:,:) + cdz(:,:)) ) ./ (abs(cdz(:,:)));
 
 % get aux metric
 l_aux = lambda(:, :) .* l(:, :);

end  